from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GetAutomotiveDealerProfileResponse200VehicleValuation")


@attr.s(auto_attribs=True)
class GetAutomotiveDealerProfileResponse200VehicleValuation:
    """
    Attributes:
        url (Union[Unset, str]):
        enabled (Union[Unset, bool]):
        threshold_in_months (Union[Unset, float]):  Example: 12.
    """

    url: Union[Unset, str] = UNSET
    enabled: Union[Unset, bool] = UNSET
    threshold_in_months: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        url = self.url
        enabled = self.enabled
        threshold_in_months = self.threshold_in_months

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if url is not UNSET:
            field_dict["url"] = url
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if threshold_in_months is not UNSET:
            field_dict["thresholdInMonths"] = threshold_in_months

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        url = d.pop("url", UNSET)

        enabled = d.pop("enabled", UNSET)

        threshold_in_months = d.pop("thresholdInMonths", UNSET)

        get_automotive_dealer_profile_response_200_vehicle_valuation = cls(
            url=url,
            enabled=enabled,
            threshold_in_months=threshold_in_months,
        )

        get_automotive_dealer_profile_response_200_vehicle_valuation.additional_properties = d
        return get_automotive_dealer_profile_response_200_vehicle_valuation

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class EventType(str, Enum):
    EXT_PWR_LOW = "EXT_PWR_LOW"
    SPEED = "SPEED"
    MOVE_START = "MOVE_START"
    MOVE_STOP = "MOVE_STOP"
    MOVE_PER = "MOVE_PER"
    TRIP_START = "TRIP_START"
    TRIP_STOP = "TRIP_STOP"
    SLEEP_ENTER = "SLEEP_ENTER"
    CFG_RESP = "CFG_RESP"
    USER_LOC = "USER_LOC"
    AUTO_LOC = "AUTO_LOC"
    GEO_ENTER = "GEO_ENTER"
    GEO_EXIT = "GEO_EXIT"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.get_maintenance_schedule_response_200_content_item_service_schedule_item_service_items import (
    GetMaintenanceScheduleResponse200ContentItemServiceScheduleItemServiceItems,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="GetMaintenanceScheduleResponse200ContentItemServiceScheduleItem")


@attr.s(auto_attribs=True)
class GetMaintenanceScheduleResponse200ContentItemServiceScheduleItem:
    """
    Attributes:
        mileage (Union[Unset, float]):  Example: 15000.
        service_items (Union[Unset, GetMaintenanceScheduleResponse200ContentItemServiceScheduleItemServiceItems]):
    """

    mileage: Union[Unset, float] = UNSET
    service_items: Union[Unset, GetMaintenanceScheduleResponse200ContentItemServiceScheduleItemServiceItems] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        mileage = self.mileage
        service_items: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.service_items, Unset):
            service_items = self.service_items.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if mileage is not UNSET:
            field_dict["mileage"] = mileage
        if service_items is not UNSET:
            field_dict["serviceItems"] = service_items

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        mileage = d.pop("mileage", UNSET)

        _service_items = d.pop("serviceItems", UNSET)
        service_items: Union[Unset, GetMaintenanceScheduleResponse200ContentItemServiceScheduleItemServiceItems]
        if isinstance(_service_items, Unset):
            service_items = UNSET
        else:
            service_items = GetMaintenanceScheduleResponse200ContentItemServiceScheduleItemServiceItems.from_dict(
                _service_items
            )

        get_maintenance_schedule_response_200_content_item_service_schedule_item = cls(
            mileage=mileage,
            service_items=service_items,
        )

        get_maintenance_schedule_response_200_content_item_service_schedule_item.additional_properties = d
        return get_maintenance_schedule_response_200_content_item_service_schedule_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.preference_key import PreferenceKey
from ..types import UNSET, Unset

T = TypeVar("T", bound="Preference")


@attr.s(auto_attribs=True)
class Preference:
    """
    Attributes:
        key (Union[Unset, PreferenceKey]):
        value (Union[Unset, bool]):
        namespace (Union[Unset, str]):  Example: consumerMobile.SkyLink.
    """

    key: Union[Unset, PreferenceKey] = UNSET
    value: Union[Unset, bool] = UNSET
    namespace: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        key: Union[Unset, str] = UNSET
        if not isinstance(self.key, Unset):
            key = self.key.value

        value = self.value
        namespace = self.namespace

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if key is not UNSET:
            field_dict["key"] = key
        if value is not UNSET:
            field_dict["value"] = value
        if namespace is not UNSET:
            field_dict["namespace"] = namespace

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _key = d.pop("key", UNSET)
        key: Union[Unset, PreferenceKey]
        if isinstance(_key, Unset):
            key = UNSET
        else:
            key = PreferenceKey(_key)

        value = d.pop("value", UNSET)

        namespace = d.pop("namespace", UNSET)

        preference = cls(
            key=key,
            value=value,
            namespace=namespace,
        )

        preference.additional_properties = d
        return preference

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.device_features_item import DeviceFeaturesItem
from ..types import UNSET, Unset

T = TypeVar("T", bound="Device")


@attr.s(auto_attribs=True)
class Device:
    """
    Attributes:
        id (Union[Unset, str]): Device ID Example: 38261738B7DG937BXMQ3.
        account_id (Union[Unset, str]):  Example: 83628362941HDHQQ63V1.
        active (Union[Unset, bool]):  Example: True.
        type (Union[Unset, str]):  Example: UNKNOWN.
        features (Union[Unset, List[DeviceFeaturesItem]]):
        serial_number (Union[Unset, str]):  Example: BSHA638QKNB1.
        date_created (Union[Unset, str]):  Example: 2022-05-10T03:59:59.999+0000.
        last_updated (Union[Unset, str]):  Example: 2022-05-10T03:59:59.999+0000.
        created_by (Union[Unset, str]):  Example: O2C.
        updated_by (Union[Unset, str]):  Example: otaService@spireon.com.
    """

    id: Union[Unset, str] = UNSET
    account_id: Union[Unset, str] = UNSET
    active: Union[Unset, bool] = UNSET
    type: Union[Unset, str] = UNSET
    features: Union[Unset, List[DeviceFeaturesItem]] = UNSET
    serial_number: Union[Unset, str] = UNSET
    date_created: Union[Unset, str] = UNSET
    last_updated: Union[Unset, str] = UNSET
    created_by: Union[Unset, str] = UNSET
    updated_by: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        account_id = self.account_id
        active = self.active
        type = self.type
        features: Union[Unset, List[str]] = UNSET
        if not isinstance(self.features, Unset):
            features = []
            for features_item_data in self.features:
                features_item = features_item_data.value

                features.append(features_item)

        serial_number = self.serial_number
        date_created = self.date_created
        last_updated = self.last_updated
        created_by = self.created_by
        updated_by = self.updated_by

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if account_id is not UNSET:
            field_dict["accountId"] = account_id
        if active is not UNSET:
            field_dict["active"] = active
        if type is not UNSET:
            field_dict["type"] = type
        if features is not UNSET:
            field_dict["features"] = features
        if serial_number is not UNSET:
            field_dict["serialNumber"] = serial_number
        if date_created is not UNSET:
            field_dict["dateCreated"] = date_created
        if last_updated is not UNSET:
            field_dict["lastUpdated"] = last_updated
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if updated_by is not UNSET:
            field_dict["updatedBy"] = updated_by

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        account_id = d.pop("accountId", UNSET)

        active = d.pop("active", UNSET)

        type = d.pop("type", UNSET)

        features = []
        _features = d.pop("features", UNSET)
        for features_item_data in _features or []:
            features_item = DeviceFeaturesItem(features_item_data)

            features.append(features_item)

        serial_number = d.pop("serialNumber", UNSET)

        date_created = d.pop("dateCreated", UNSET)

        last_updated = d.pop("lastUpdated", UNSET)

        created_by = d.pop("createdBy", UNSET)

        updated_by = d.pop("updatedBy", UNSET)

        device = cls(
            id=id,
            account_id=account_id,
            active=active,
            type=type,
            features=features,
            serial_number=serial_number,
            date_created=date_created,
            last_updated=last_updated,
            created_by=created_by,
            updated_by=updated_by,
        )

        device.additional_properties = d
        return device

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .get_maintenance_schedule_response_200 import GetMaintenanceScheduleResponse200
from .get_maintenance_schedule_response_200_content_item import GetMaintenanceScheduleResponse200ContentItem
from .get_maintenance_schedule_response_200_content_item_service_schedule_item import (
    GetMaintenanceScheduleResponse200ContentItemServiceScheduleItem,
)
from .get_maintenance_schedule_response_200_content_item_service_schedule_item_service_items import (
    GetMaintenanceScheduleResponse200ContentItemServiceScheduleItemServiceItems,
)
from .get_maintenance_schedule_response_200_content_item_service_schedule_item_service_items_severity import (
    GetMaintenanceScheduleResponse200ContentItemServiceScheduleItemServiceItemsSeverity,
)
from .get_repair_orders_response_200 import GetRepairOrdersResponse200
from .get_repair_orders_response_200_content_item import GetRepairOrdersResponse200ContentItem



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.get_maintenance_schedule_response_200_content_item import GetMaintenanceScheduleResponse200ContentItem
from ..types import UNSET, Unset

T = TypeVar("T", bound="GetMaintenanceScheduleResponse200")


@attr.s(auto_attribs=True)
class GetMaintenanceScheduleResponse200:
    """
    Attributes:
        content (Union[Unset, List[GetMaintenanceScheduleResponse200ContentItem]]):
        count (Union[Unset, float]):  Example: 1.
        total (Union[Unset, float]):  Example: 1.
    """

    content: Union[Unset, List[GetMaintenanceScheduleResponse200ContentItem]] = UNSET
    count: Union[Unset, float] = UNSET
    total: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        content: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.content, Unset):
            content = []
            for content_item_data in self.content:
                content_item = content_item_data.to_dict()

                content.append(content_item)

        count = self.count
        total = self.total

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if content is not UNSET:
            field_dict["content"] = content
        if count is not UNSET:
            field_dict["count"] = count
        if total is not UNSET:
            field_dict["total"] = total

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        content = []
        _content = d.pop("content", UNSET)
        for content_item_data in _content or []:
            content_item = GetMaintenanceScheduleResponse200ContentItem.from_dict(content_item_data)

            content.append(content_item)

        count = d.pop("count", UNSET)

        total = d.pop("total", UNSET)

        get_maintenance_schedule_response_200 = cls(
            content=content,
            count=count,
            total=total,
        )

        get_maintenance_schedule_response_200.additional_properties = d
        return get_maintenance_schedule_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Address")


@attr.s(auto_attribs=True)
class Address:
    """
    Attributes:
        line1 (Union[Unset, str]):  Example: 123 Street Name.
        city (Union[Unset, str]):  Example: City Name.
        state_or_province (Union[Unset, str]):  Example: ST.
        postal_code (Union[Unset, str]):  Example: 30098.
    """

    line1: Union[Unset, str] = UNSET
    city: Union[Unset, str] = UNSET
    state_or_province: Union[Unset, str] = UNSET
    postal_code: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        line1 = self.line1
        city = self.city
        state_or_province = self.state_or_province
        postal_code = self.postal_code

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if line1 is not UNSET:
            field_dict["line1"] = line1
        if city is not UNSET:
            field_dict["city"] = city
        if state_or_province is not UNSET:
            field_dict["stateOrProvince"] = state_or_province
        if postal_code is not UNSET:
            field_dict["postalCode"] = postal_code

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        line1 = d.pop("line1", UNSET)

        city = d.pop("city", UNSET)

        state_or_province = d.pop("stateOrProvince", UNSET)

        postal_code = d.pop("postalCode", UNSET)

        address = cls(
            line1=line1,
            city=city,
            state_or_province=state_or_province,
            postal_code=postal_code,
        )

        address.additional_properties = d
        return address

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="AssetInstrumentationsResponse200AssetRef")


@attr.s(auto_attribs=True)
class AssetInstrumentationsResponse200AssetRef:
    """
    Attributes:
        id (Union[Unset, str]): Asset ID Example: 272956057382HD3JBSD24.
        name (Union[Unset, str]):  Example: Consumer Asset VINNUMBER12356,.
        vin (Union[Unset, str]):  Example: VINNUMBER12345.
        href (Union[Unset, str]):  Example: /assets/272956057382HD3JBSD24.
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    vin: Union[Unset, str] = UNSET
    href: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        vin = self.vin
        href = self.href

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if vin is not UNSET:
            field_dict["vin"] = vin
        if href is not UNSET:
            field_dict["href"] = href

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        vin = d.pop("vin", UNSET)

        href = d.pop("href", UNSET)

        asset_instrumentations_response_200_asset_ref = cls(
            id=id,
            name=name,
            vin=vin,
            href=href,
        )

        asset_instrumentations_response_200_asset_ref.additional_properties = d
        return asset_instrumentations_response_200_asset_ref

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.command_request_response_201_command import CommandRequestResponse201Command
from ..models.command_request_response_201_response_strategy import CommandRequestResponse201ResponseStrategy
from ..types import UNSET, Unset

T = TypeVar("T", bound="CommandRequestResponse201")


@attr.s(auto_attribs=True)
class CommandRequestResponse201:
    """
    Attributes:
        id (Union[Unset, str]):  Example: 436tef1637vd7623g1.
        command (Union[Unset, CommandRequestResponse201Command]):  Example: locate.
        device_id (Union[Unset, str]): Device ID Example: 38261738B7DG937BXMQ3.
        asset_id (Union[Unset, str]): Asset ID Example: 272956057382HD3JBSD24.
        response_channel (Union[Unset, str]):  Example: consumerMobile_assetId_272956057382HD3JBSD24.
        response_strategy (Union[Unset, CommandRequestResponse201ResponseStrategy]):  Example: PUSH_NOTIFICATION.
        requested_by (Union[Unset, str]):  Example: username31.
    """

    id: Union[Unset, str] = UNSET
    command: Union[Unset, CommandRequestResponse201Command] = UNSET
    device_id: Union[Unset, str] = UNSET
    asset_id: Union[Unset, str] = UNSET
    response_channel: Union[Unset, str] = UNSET
    response_strategy: Union[Unset, CommandRequestResponse201ResponseStrategy] = UNSET
    requested_by: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        command: Union[Unset, str] = UNSET
        if not isinstance(self.command, Unset):
            command = self.command.value

        device_id = self.device_id
        asset_id = self.asset_id
        response_channel = self.response_channel
        response_strategy: Union[Unset, str] = UNSET
        if not isinstance(self.response_strategy, Unset):
            response_strategy = self.response_strategy.value

        requested_by = self.requested_by

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if command is not UNSET:
            field_dict["command"] = command
        if device_id is not UNSET:
            field_dict["deviceId"] = device_id
        if asset_id is not UNSET:
            field_dict["assetId"] = asset_id
        if response_channel is not UNSET:
            field_dict["responseChannel"] = response_channel
        if response_strategy is not UNSET:
            field_dict["responseStrategy"] = response_strategy
        if requested_by is not UNSET:
            field_dict["requestedBy"] = requested_by

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        _command = d.pop("command", UNSET)
        command: Union[Unset, CommandRequestResponse201Command]
        if isinstance(_command, Unset):
            command = UNSET
        else:
            command = CommandRequestResponse201Command(_command)

        device_id = d.pop("deviceId", UNSET)

        asset_id = d.pop("assetId", UNSET)

        response_channel = d.pop("responseChannel", UNSET)

        _response_strategy = d.pop("responseStrategy", UNSET)
        response_strategy: Union[Unset, CommandRequestResponse201ResponseStrategy]
        if isinstance(_response_strategy, Unset):
            response_strategy = UNSET
        else:
            response_strategy = CommandRequestResponse201ResponseStrategy(_response_strategy)

        requested_by = d.pop("requestedBy", UNSET)

        command_request_response_201 = cls(
            id=id,
            command=command,
            device_id=device_id,
            asset_id=asset_id,
            response_channel=response_channel,
            response_strategy=response_strategy,
            requested_by=requested_by,
        )

        command_request_response_201.additional_properties = d
        return command_request_response_201

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GetTermsOfServiceResponse200")


@attr.s(auto_attribs=True)
class GetTermsOfServiceResponse200:
    """
    Attributes:
        id (Union[Unset, str]):  Example: SBDGFVBAGWGVQ352521.
        name (Union[Unset, str]):  Example: LoJack SSA.
        content (Union[Unset, str]): Long HTML output with terms of service
        accepted (Union[Unset, bool]):  Example: True.
        can_accept (Union[Unset, bool]):
        date_created (Union[Unset, str]):  Example: 2022-05-10T03:59:59.999+0000.
        last_updated (Union[Unset, str]):  Example: 2022-05-10T03:59:59.999+0000.
        created_by (Union[Unset, str]):  Example: email@spireon.com.
        ttl_days (Union[Unset, float]):
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    content: Union[Unset, str] = UNSET
    accepted: Union[Unset, bool] = UNSET
    can_accept: Union[Unset, bool] = UNSET
    date_created: Union[Unset, str] = UNSET
    last_updated: Union[Unset, str] = UNSET
    created_by: Union[Unset, str] = UNSET
    ttl_days: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        content = self.content
        accepted = self.accepted
        can_accept = self.can_accept
        date_created = self.date_created
        last_updated = self.last_updated
        created_by = self.created_by
        ttl_days = self.ttl_days

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if content is not UNSET:
            field_dict["content"] = content
        if accepted is not UNSET:
            field_dict["accepted"] = accepted
        if can_accept is not UNSET:
            field_dict["canAccept"] = can_accept
        if date_created is not UNSET:
            field_dict["dateCreated"] = date_created
        if last_updated is not UNSET:
            field_dict["lastUpdated"] = last_updated
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if ttl_days is not UNSET:
            field_dict["ttlDays"] = ttl_days

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        content = d.pop("content", UNSET)

        accepted = d.pop("accepted", UNSET)

        can_accept = d.pop("canAccept", UNSET)

        date_created = d.pop("dateCreated", UNSET)

        last_updated = d.pop("lastUpdated", UNSET)

        created_by = d.pop("createdBy", UNSET)

        ttl_days = d.pop("ttlDays", UNSET)

        get_terms_of_service_response_200 = cls(
            id=id,
            name=name,
            content=content,
            accepted=accepted,
            can_accept=can_accept,
            date_created=date_created,
            last_updated=last_updated,
            created_by=created_by,
            ttl_days=ttl_days,
        )

        get_terms_of_service_response_200.additional_properties = d
        return get_terms_of_service_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GetIdentityTokenResponse200")


@attr.s(auto_attribs=True)
class GetIdentityTokenResponse200:
    """
    Attributes:
        token (Union[Unset, str]):  Example: {jwt}.
        scope (Union[Unset, str]): scope Example: ACCOUNT_USER_SCOPE.
        expires_on (Union[Unset, str]):  Example: 2022-05-10T03:59:59.999+0000.
        refresh_by (Union[Unset, str]):  Example: 2022-05-10T03:59:59.999+0000.
        refresh_in_seconds (Union[Unset, float]):  Example: 86340.
    """

    token: Union[Unset, str] = UNSET
    scope: Union[Unset, str] = UNSET
    expires_on: Union[Unset, str] = UNSET
    refresh_by: Union[Unset, str] = UNSET
    refresh_in_seconds: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        token = self.token
        scope = self.scope
        expires_on = self.expires_on
        refresh_by = self.refresh_by
        refresh_in_seconds = self.refresh_in_seconds

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if token is not UNSET:
            field_dict["token"] = token
        if scope is not UNSET:
            field_dict["scope"] = scope
        if expires_on is not UNSET:
            field_dict["expiresOn"] = expires_on
        if refresh_by is not UNSET:
            field_dict["refreshBy"] = refresh_by
        if refresh_in_seconds is not UNSET:
            field_dict["refreshInSeconds"] = refresh_in_seconds

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        token = d.pop("token", UNSET)

        scope = d.pop("scope", UNSET)

        expires_on = d.pop("expiresOn", UNSET)

        refresh_by = d.pop("refreshBy", UNSET)

        refresh_in_seconds = d.pop("refreshInSeconds", UNSET)

        get_identity_token_response_200 = cls(
            token=token,
            scope=scope,
            expires_on=expires_on,
            refresh_by=refresh_by,
            refresh_in_seconds=refresh_in_seconds,
        )

        get_identity_token_response_200.additional_properties = d
        return get_identity_token_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.update_asset_json_body_attributes import UpdateAssetJsonBodyAttributes
from ..types import UNSET, Unset

T = TypeVar("T", bound="UpdateAssetJsonBody")


@attr.s(auto_attribs=True)
class UpdateAssetJsonBody:
    """
    Attributes:
        attributes (Union[Unset, UpdateAssetJsonBodyAttributes]):
        id (Union[Unset, str]): Asset ID Example: 272956057382HD3JBSD24.
        make (Union[Unset, str]):  Example: Ford.
        model (Union[Unset, str]):  Example: Explorer.
        name (Union[Unset, str]):  Example: Consumer Asset VINNUMBER12345.
        odometer (Union[Unset, str]):  Example: 519.89.
        vin (Union[Unset, str]):  Example: VINNUMBER12345.
        year (Union[Unset, str]):  Example: 2022.
    """

    attributes: Union[Unset, UpdateAssetJsonBodyAttributes] = UNSET
    id: Union[Unset, str] = UNSET
    make: Union[Unset, str] = UNSET
    model: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    odometer: Union[Unset, str] = UNSET
    vin: Union[Unset, str] = UNSET
    year: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        attributes: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.attributes, Unset):
            attributes = self.attributes.to_dict()

        id = self.id
        make = self.make
        model = self.model
        name = self.name
        odometer = self.odometer
        vin = self.vin
        year = self.year

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if attributes is not UNSET:
            field_dict["attributes"] = attributes
        if id is not UNSET:
            field_dict["id"] = id
        if make is not UNSET:
            field_dict["make"] = make
        if model is not UNSET:
            field_dict["model"] = model
        if name is not UNSET:
            field_dict["name"] = name
        if odometer is not UNSET:
            field_dict["odometer"] = odometer
        if vin is not UNSET:
            field_dict["vin"] = vin
        if year is not UNSET:
            field_dict["year"] = year

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _attributes = d.pop("attributes", UNSET)
        attributes: Union[Unset, UpdateAssetJsonBodyAttributes]
        if isinstance(_attributes, Unset):
            attributes = UNSET
        else:
            attributes = UpdateAssetJsonBodyAttributes.from_dict(_attributes)

        id = d.pop("id", UNSET)

        make = d.pop("make", UNSET)

        model = d.pop("model", UNSET)

        name = d.pop("name", UNSET)

        odometer = d.pop("odometer", UNSET)

        vin = d.pop("vin", UNSET)

        year = d.pop("year", UNSET)

        update_asset_json_body = cls(
            attributes=attributes,
            id=id,
            make=make,
            model=model,
            name=name,
            odometer=odometer,
            vin=vin,
            year=year,
        )

        update_asset_json_body.additional_properties = d
        return update_asset_json_body

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.event_gps_fix_quality import EventGpsFixQuality
from ..models.event_type import EventType
from ..models.geofence_event_data import GeofenceEventData
from ..models.location import Location
from ..models.null_value import NullValue
from ..types import UNSET, Unset

T = TypeVar("T", bound="Event")


@attr.s(auto_attribs=True)
class Event:
    """
    Attributes:
        id (Union[Unset, str]):  Example: 9085a727-a00f-4ee6-9363-603a6266b0d5.
        type (Union[Unset, EventType]):
        location (Union[Unset, Location]):
        date (Union[Unset, str]):  Example: 2022-05-10T03:59:59.999+0000.
        odometer (Union[Unset, float]):  Example: 324.19.
        battery_voltage (Union[Unset, float]):  Example: 15.124.
        heading (Union[Unset, int]):  Example: 250.
        speed (Union[Unset, int]):
        engine_hours (Union[Unset, int]):
        distance_driven (Union[Unset, float]):  Example: 724.241.
        signal_strength (Union[Unset, float]):  Example: 0.815.
        gps_fix_quality (Union[Unset, EventGpsFixQuality]):
        event_data (Union[GeofenceEventData, NullValue, Unset]):
    """

    id: Union[Unset, str] = UNSET
    type: Union[Unset, EventType] = UNSET
    location: Union[Unset, Location] = UNSET
    date: Union[Unset, str] = UNSET
    odometer: Union[Unset, float] = UNSET
    battery_voltage: Union[Unset, float] = UNSET
    heading: Union[Unset, int] = UNSET
    speed: Union[Unset, int] = UNSET
    engine_hours: Union[Unset, int] = UNSET
    distance_driven: Union[Unset, float] = UNSET
    signal_strength: Union[Unset, float] = UNSET
    gps_fix_quality: Union[Unset, EventGpsFixQuality] = UNSET
    event_data: Union[GeofenceEventData, NullValue, Unset] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        location: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.location, Unset):
            location = self.location.to_dict()

        date = self.date
        odometer = self.odometer
        battery_voltage = self.battery_voltage
        heading = self.heading
        speed = self.speed
        engine_hours = self.engine_hours
        distance_driven = self.distance_driven
        signal_strength = self.signal_strength
        gps_fix_quality: Union[Unset, str] = UNSET
        if not isinstance(self.gps_fix_quality, Unset):
            gps_fix_quality = self.gps_fix_quality.value

        event_data: Union[Dict[str, Any], Unset, str]
        if isinstance(self.event_data, Unset):
            event_data = UNSET

        elif isinstance(self.event_data, GeofenceEventData):
            event_data = UNSET
            if not isinstance(self.event_data, Unset):
                event_data = self.event_data.to_dict()

        else:
            event_data = UNSET
            if not isinstance(self.event_data, Unset):
                event_data = self.event_data.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["type"] = type
        if location is not UNSET:
            field_dict["location"] = location
        if date is not UNSET:
            field_dict["date"] = date
        if odometer is not UNSET:
            field_dict["odometer"] = odometer
        if battery_voltage is not UNSET:
            field_dict["batteryVoltage"] = battery_voltage
        if heading is not UNSET:
            field_dict["heading"] = heading
        if speed is not UNSET:
            field_dict["speed"] = speed
        if engine_hours is not UNSET:
            field_dict["engineHours"] = engine_hours
        if distance_driven is not UNSET:
            field_dict["distanceDriven"] = distance_driven
        if signal_strength is not UNSET:
            field_dict["signalStrength"] = signal_strength
        if gps_fix_quality is not UNSET:
            field_dict["gpsFixQuality"] = gps_fix_quality
        if event_data is not UNSET:
            field_dict["eventData"] = event_data

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, EventType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = EventType(_type)

        _location = d.pop("location", UNSET)
        location: Union[Unset, Location]
        if isinstance(_location, Unset):
            location = UNSET
        else:
            location = Location.from_dict(_location)

        date = d.pop("date", UNSET)

        odometer = d.pop("odometer", UNSET)

        battery_voltage = d.pop("batteryVoltage", UNSET)

        heading = d.pop("heading", UNSET)

        speed = d.pop("speed", UNSET)

        engine_hours = d.pop("engineHours", UNSET)

        distance_driven = d.pop("distanceDriven", UNSET)

        signal_strength = d.pop("signalStrength", UNSET)

        _gps_fix_quality = d.pop("gpsFixQuality", UNSET)
        gps_fix_quality: Union[Unset, EventGpsFixQuality]
        if isinstance(_gps_fix_quality, Unset):
            gps_fix_quality = UNSET
        else:
            gps_fix_quality = EventGpsFixQuality(_gps_fix_quality)

        def _parse_event_data(data: object) -> Union[GeofenceEventData, NullValue, Unset]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                _event_data_type_0 = data
                event_data_type_0: Union[Unset, GeofenceEventData]
                if isinstance(_event_data_type_0, Unset):
                    event_data_type_0 = UNSET
                else:
                    event_data_type_0 = GeofenceEventData.from_dict(_event_data_type_0)

                return event_data_type_0
            except:  # noqa: E722
                pass
            if not isinstance(data, str):
                raise TypeError()
            _event_data_type_1 = data
            event_data_type_1: Union[Unset, NullValue]
            if isinstance(_event_data_type_1, Unset):
                event_data_type_1 = UNSET
            else:
                event_data_type_1 = NullValue(_event_data_type_1)

            return event_data_type_1

        event_data = _parse_event_data(d.pop("eventData", UNSET))

        event = cls(
            id=id,
            type=type,
            location=location,
            date=date,
            odometer=odometer,
            battery_voltage=battery_voltage,
            heading=heading,
            speed=speed,
            engine_hours=engine_hours,
            distance_driven=distance_driven,
            signal_strength=signal_strength,
            gps_fix_quality=gps_fix_quality,
            event_data=event_data,
        )

        event.additional_properties = d
        return event

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import Client
from ...models.get_all_user_assets_response_200 import GetAllUserAssetsResponse200
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    id: Union[Unset, None, str] = UNSET,
    color: Union[Unset, None, str] = UNSET,
    vin: Union[Unset, None, str] = UNSET,
    year: Union[Unset, None, str] = UNSET,
    make: Union[Unset, None, str] = UNSET,
    model: Union[Unset, None, str] = UNSET,
    status: Union[Unset, None, str] = UNSET,
    active: Union[Unset, None, bool] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = UNSET,
) -> Dict[str, Any]:
    url = "{}/assets".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["id"] = id

    params["color"] = color

    params["vin"] = vin

    params["year"] = year

    params["make"] = make

    params["model"] = model

    params["status"] = status

    params["active"] = active

    params["limit"] = limit

    params["offset"] = offset

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[Any, GetAllUserAssetsResponse200]]:
    if response.status_code == 200:
        response_200 = GetAllUserAssetsResponse200.from_dict(response.json())

        return response_200
    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[Any, GetAllUserAssetsResponse200]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    id: Union[Unset, None, str] = UNSET,
    color: Union[Unset, None, str] = UNSET,
    vin: Union[Unset, None, str] = UNSET,
    year: Union[Unset, None, str] = UNSET,
    make: Union[Unset, None, str] = UNSET,
    model: Union[Unset, None, str] = UNSET,
    status: Union[Unset, None, str] = UNSET,
    active: Union[Unset, None, bool] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = UNSET,
) -> Response[Union[Any, GetAllUserAssetsResponse200]]:
    """Get all user assets

    Args:
        id (Union[Unset, None, str]): Asset ID Example: 272956057382HD3JBSD24.
        color (Union[Unset, None, str]):  Example: black.
        vin (Union[Unset, None, str]):  Example: VINNUMBER12345.
        year (Union[Unset, None, str]):  Example: 2022.
        make (Union[Unset, None, str]):  Example: Ford.
        model (Union[Unset, None, str]):  Example: Explorer.
        status (Union[Unset, None, str]):  Example: Stopped.
        active (Union[Unset, None, bool]):
        limit (Union[Unset, None, int]): Limit of results to show Example: 100.
        offset (Union[Unset, None, int]): Result offset used for pagination

    Returns:
        Response[Union[Any, GetAllUserAssetsResponse200]]
    """

    kwargs = _get_kwargs(
        client=client,
        id=id,
        color=color,
        vin=vin,
        year=year,
        make=make,
        model=model,
        status=status,
        active=active,
        limit=limit,
        offset=offset,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    id: Union[Unset, None, str] = UNSET,
    color: Union[Unset, None, str] = UNSET,
    vin: Union[Unset, None, str] = UNSET,
    year: Union[Unset, None, str] = UNSET,
    make: Union[Unset, None, str] = UNSET,
    model: Union[Unset, None, str] = UNSET,
    status: Union[Unset, None, str] = UNSET,
    active: Union[Unset, None, bool] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = UNSET,
) -> Optional[Union[Any, GetAllUserAssetsResponse200]]:
    """Get all user assets

    Args:
        id (Union[Unset, None, str]): Asset ID Example: 272956057382HD3JBSD24.
        color (Union[Unset, None, str]):  Example: black.
        vin (Union[Unset, None, str]):  Example: VINNUMBER12345.
        year (Union[Unset, None, str]):  Example: 2022.
        make (Union[Unset, None, str]):  Example: Ford.
        model (Union[Unset, None, str]):  Example: Explorer.
        status (Union[Unset, None, str]):  Example: Stopped.
        active (Union[Unset, None, bool]):
        limit (Union[Unset, None, int]): Limit of results to show Example: 100.
        offset (Union[Unset, None, int]): Result offset used for pagination

    Returns:
        Response[Union[Any, GetAllUserAssetsResponse200]]
    """

    return sync_detailed(
        client=client,
        id=id,
        color=color,
        vin=vin,
        year=year,
        make=make,
        model=model,
        status=status,
        active=active,
        limit=limit,
        offset=offset,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    id: Union[Unset, None, str] = UNSET,
    color: Union[Unset, None, str] = UNSET,
    vin: Union[Unset, None, str] = UNSET,
    year: Union[Unset, None, str] = UNSET,
    make: Union[Unset, None, str] = UNSET,
    model: Union[Unset, None, str] = UNSET,
    status: Union[Unset, None, str] = UNSET,
    active: Union[Unset, None, bool] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = UNSET,
) -> Response[Union[Any, GetAllUserAssetsResponse200]]:
    """Get all user assets

    Args:
        id (Union[Unset, None, str]): Asset ID Example: 272956057382HD3JBSD24.
        color (Union[Unset, None, str]):  Example: black.
        vin (Union[Unset, None, str]):  Example: VINNUMBER12345.
        year (Union[Unset, None, str]):  Example: 2022.
        make (Union[Unset, None, str]):  Example: Ford.
        model (Union[Unset, None, str]):  Example: Explorer.
        status (Union[Unset, None, str]):  Example: Stopped.
        active (Union[Unset, None, bool]):
        limit (Union[Unset, None, int]): Limit of results to show Example: 100.
        offset (Union[Unset, None, int]): Result offset used for pagination

    Returns:
        Response[Union[Any, GetAllUserAssetsResponse200]]
    """

    kwargs = _get_kwargs(
        client=client,
        id=id,
        color=color,
        vin=vin,
        year=year,
        make=make,
        model=model,
        status=status,
        active=active,
        limit=limit,
        offset=offset,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    id: Union[Unset, None, str] = UNSET,
    color: Union[Unset, None, str] = UNSET,
    vin: Union[Unset, None, str] = UNSET,
    year: Union[Unset, None, str] = UNSET,
    make: Union[Unset, None, str] = UNSET,
    model: Union[Unset, None, str] = UNSET,
    status: Union[Unset, None, str] = UNSET,
    active: Union[Unset, None, bool] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = UNSET,
) -> Optional[Union[Any, GetAllUserAssetsResponse200]]:
    """Get all user assets

    Args:
        id (Union[Unset, None, str]): Asset ID Example: 272956057382HD3JBSD24.
        color (Union[Unset, None, str]):  Example: black.
        vin (Union[Unset, None, str]):  Example: VINNUMBER12345.
        year (Union[Unset, None, str]):  Example: 2022.
        make (Union[Unset, None, str]):  Example: Ford.
        model (Union[Unset, None, str]):  Example: Explorer.
        status (Union[Unset, None, str]):  Example: Stopped.
        active (Union[Unset, None, bool]):
        limit (Union[Unset, None, int]): Limit of results to show Example: 100.
        offset (Union[Unset, None, int]): Result offset used for pagination

    Returns:
        Response[Union[Any, GetAllUserAssetsResponse200]]
    """

    return (
        await asyncio_detailed(
            client=client,
            id=id,
            color=color,
            vin=vin,
            year=year,
            make=make,
            model=model,
            status=status,
            active=active,
            limit=limit,
            offset=offset,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.geofence_shape import GeofenceShape
from ..types import UNSET, Unset

T = TypeVar("T", bound="Geofence")


@attr.s(auto_attribs=True)
class Geofence:
    """
    Attributes:
        id (Union[Unset, str]):  Example: 1140762987615GIDWGA.
        name (Union[Unset, str]):  Example: My Home.
        shape (Union[Unset, GeofenceShape]):
        date_created (Union[Unset, str]):  Example: 2022-05-10T03:59:59.999+0000.
        last_updated (Union[Unset, str]):  Example: 2022-05-10T03:59:59.999+0000.
        created_by (Union[Unset, str]):  Example: username31.
        updated_by (Union[Unset, str]):  Example: username31.
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    shape: Union[Unset, GeofenceShape] = UNSET
    date_created: Union[Unset, str] = UNSET
    last_updated: Union[Unset, str] = UNSET
    created_by: Union[Unset, str] = UNSET
    updated_by: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        shape: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.shape, Unset):
            shape = self.shape.to_dict()

        date_created = self.date_created
        last_updated = self.last_updated
        created_by = self.created_by
        updated_by = self.updated_by

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if shape is not UNSET:
            field_dict["shape"] = shape
        if date_created is not UNSET:
            field_dict["dateCreated"] = date_created
        if last_updated is not UNSET:
            field_dict["lastUpdated"] = last_updated
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if updated_by is not UNSET:
            field_dict["updatedBy"] = updated_by

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        _shape = d.pop("shape", UNSET)
        shape: Union[Unset, GeofenceShape]
        if isinstance(_shape, Unset):
            shape = UNSET
        else:
            shape = GeofenceShape.from_dict(_shape)

        date_created = d.pop("dateCreated", UNSET)

        last_updated = d.pop("lastUpdated", UNSET)

        created_by = d.pop("createdBy", UNSET)

        updated_by = d.pop("updatedBy", UNSET)

        geofence = cls(
            id=id,
            name=name,
            shape=shape,
            date_created=date_created,
            last_updated=last_updated,
            created_by=created_by,
            updated_by=updated_by,
        )

        geofence.additional_properties = d
        return geofence

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.get_maintenance_schedule_response_200_content_item_service_schedule_item import (
    GetMaintenanceScheduleResponse200ContentItemServiceScheduleItem,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="GetMaintenanceScheduleResponse200ContentItem")


@attr.s(auto_attribs=True)
class GetMaintenanceScheduleResponse200ContentItem:
    """
    Attributes:
        id (Union[Unset, str]):  Example: 8e4cd954-41a3-4a1f-98d4-ab6b306614d9.
        service_interval (Union[Unset, float]): The mileage interval at which services are recommend (eg. new services
            every 5000 miles) Example: 5000.
        service_schedule (Union[Unset, List[GetMaintenanceScheduleResponse200ContentItemServiceScheduleItem]]):
    """

    id: Union[Unset, str] = UNSET
    service_interval: Union[Unset, float] = UNSET
    service_schedule: Union[Unset, List[GetMaintenanceScheduleResponse200ContentItemServiceScheduleItem]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        service_interval = self.service_interval
        service_schedule: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.service_schedule, Unset):
            service_schedule = []
            for service_schedule_item_data in self.service_schedule:
                service_schedule_item = service_schedule_item_data.to_dict()

                service_schedule.append(service_schedule_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if service_interval is not UNSET:
            field_dict["serviceInterval"] = service_interval
        if service_schedule is not UNSET:
            field_dict["serviceSchedule"] = service_schedule

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        service_interval = d.pop("serviceInterval", UNSET)

        service_schedule = []
        _service_schedule = d.pop("serviceSchedule", UNSET)
        for service_schedule_item_data in _service_schedule or []:
            service_schedule_item = GetMaintenanceScheduleResponse200ContentItemServiceScheduleItem.from_dict(
                service_schedule_item_data
            )

            service_schedule.append(service_schedule_item)

        get_maintenance_schedule_response_200_content_item = cls(
            id=id,
            service_interval=service_interval,
            service_schedule=service_schedule,
        )

        get_maintenance_schedule_response_200_content_item.additional_properties = d
        return get_maintenance_schedule_response_200_content_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.command_request_json_body_command import CommandRequestJsonBodyCommand
from ..models.command_request_json_body_response_strategy import CommandRequestJsonBodyResponseStrategy
from ..types import UNSET, Unset

T = TypeVar("T", bound="CommandRequestJsonBody")


@attr.s(auto_attribs=True)
class CommandRequestJsonBody:
    """
    Attributes:
        asset_id (Union[Unset, str]): Asset ID Example: 272956057382HD3JBSD24.
        command (Union[Unset, CommandRequestJsonBodyCommand]):  Example: locate.
        response_channel (Union[Unset, str]):  Example: consumerMobile_assetId_272956057382HD3JBSD24.
        response_strategy (Union[Unset, CommandRequestJsonBodyResponseStrategy]):  Example: PUSH_NOTIFICATION.
    """

    asset_id: Union[Unset, str] = UNSET
    command: Union[Unset, CommandRequestJsonBodyCommand] = UNSET
    response_channel: Union[Unset, str] = UNSET
    response_strategy: Union[Unset, CommandRequestJsonBodyResponseStrategy] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        asset_id = self.asset_id
        command: Union[Unset, str] = UNSET
        if not isinstance(self.command, Unset):
            command = self.command.value

        response_channel = self.response_channel
        response_strategy: Union[Unset, str] = UNSET
        if not isinstance(self.response_strategy, Unset):
            response_strategy = self.response_strategy.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if asset_id is not UNSET:
            field_dict["assetId"] = asset_id
        if command is not UNSET:
            field_dict["command"] = command
        if response_channel is not UNSET:
            field_dict["responseChannel"] = response_channel
        if response_strategy is not UNSET:
            field_dict["responseStrategy"] = response_strategy

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        asset_id = d.pop("assetId", UNSET)

        _command = d.pop("command", UNSET)
        command: Union[Unset, CommandRequestJsonBodyCommand]
        if isinstance(_command, Unset):
            command = UNSET
        else:
            command = CommandRequestJsonBodyCommand(_command)

        response_channel = d.pop("responseChannel", UNSET)

        _response_strategy = d.pop("responseStrategy", UNSET)
        response_strategy: Union[Unset, CommandRequestJsonBodyResponseStrategy]
        if isinstance(_response_strategy, Unset):
            response_strategy = UNSET
        else:
            response_strategy = CommandRequestJsonBodyResponseStrategy(_response_strategy)

        command_request_json_body = cls(
            asset_id=asset_id,
            command=command,
            response_channel=response_channel,
            response_strategy=response_strategy,
        )

        command_request_json_body.additional_properties = d
        return command_request_json_body

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class PreferenceKey(str, Enum):
    APPTHEME = "appTheme"
    GEOFENCEALERT_EMAIL = "GeofenceAlert.email"
    GEOFENCEALERT_PUSHNOTIFICATION = "GeofenceAlert.pushNotification"
    LOWBATTERYALERT_EMAIL = "LowBatteryAlert.email"
    LOWBATTERYALERT_PUSHNOTIFICATION = "LowBatteryAlert.pushNotification"
    SERVICEALERTS_EMAIL = "ServiceAlerts.email"
    SERVICEALERTS_PUSHNOTIFICATION = "ServiceAlerts.pushNotification"
    SPEEDALERT_EMAIL = "SpeedAlert.email"
    SPEEDALERT_PUSHNOTIFICATION = "SpeedAlert.pushNotification"

    def __str__(self) -> str:
        return str(self.value)



""" Contains all the data models used in inputs/outputs """

from .get_automotive_dealer_profile_response_200 import GetAutomotiveDealerProfileResponse200
from .get_automotive_dealer_profile_response_200_dealership_address import (
    GetAutomotiveDealerProfileResponse200DealershipAddress,
)
from .get_automotive_dealer_profile_response_200_vehicle_valuation import (
    GetAutomotiveDealerProfileResponse200VehicleValuation,
)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GetUserInformationResponse200User")


@attr.s(auto_attribs=True)
class GetUserInformationResponse200User:
    """
    Attributes:
        id (Union[Unset, float]):  Example: 1854583.
        global_id (Union[Unset, str]):  Example: 9085a727-a00f-4ee6-9363-603a6266b0d5.
        username (Union[Unset, str]):  Example: username31.
        first_name (Union[Unset, str]):  Example: John,.
        last_name (Union[Unset, str]):  Example: Doe,.
        email (Union[Unset, str]): Your email address Example: email@provider.com.
        phone (Union[Unset, str]): Your phone number Example: (502) 820-1852.
        date_created (Union[Unset, str]):  Example: 2022-05-10T03:59:59.999+0000.
        last_updated (Union[Unset, str]):  Example: 2022-05-10T03:59:59.999+0000.
        created_by (Union[Unset, str]):  Example: InvitationAuthenticationProvider.
        updated_by (Union[Unset, str]):  Example: username31.
        active (Union[Unset, bool]):  Example: True.
    """

    id: Union[Unset, float] = UNSET
    global_id: Union[Unset, str] = UNSET
    username: Union[Unset, str] = UNSET
    first_name: Union[Unset, str] = UNSET
    last_name: Union[Unset, str] = UNSET
    email: Union[Unset, str] = UNSET
    phone: Union[Unset, str] = UNSET
    date_created: Union[Unset, str] = UNSET
    last_updated: Union[Unset, str] = UNSET
    created_by: Union[Unset, str] = UNSET
    updated_by: Union[Unset, str] = UNSET
    active: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        global_id = self.global_id
        username = self.username
        first_name = self.first_name
        last_name = self.last_name
        email = self.email
        phone = self.phone
        date_created = self.date_created
        last_updated = self.last_updated
        created_by = self.created_by
        updated_by = self.updated_by
        active = self.active

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if global_id is not UNSET:
            field_dict["globalId"] = global_id
        if username is not UNSET:
            field_dict["username"] = username
        if first_name is not UNSET:
            field_dict["firstName"] = first_name
        if last_name is not UNSET:
            field_dict["lastName"] = last_name
        if email is not UNSET:
            field_dict["email"] = email
        if phone is not UNSET:
            field_dict["phone"] = phone
        if date_created is not UNSET:
            field_dict["dateCreated"] = date_created
        if last_updated is not UNSET:
            field_dict["lastUpdated"] = last_updated
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if updated_by is not UNSET:
            field_dict["updatedBy"] = updated_by
        if active is not UNSET:
            field_dict["active"] = active

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        global_id = d.pop("globalId", UNSET)

        username = d.pop("username", UNSET)

        first_name = d.pop("firstName", UNSET)

        last_name = d.pop("lastName", UNSET)

        email = d.pop("email", UNSET)

        phone = d.pop("phone", UNSET)

        date_created = d.pop("dateCreated", UNSET)

        last_updated = d.pop("lastUpdated", UNSET)

        created_by = d.pop("createdBy", UNSET)

        updated_by = d.pop("updatedBy", UNSET)

        active = d.pop("active", UNSET)

        get_user_information_response_200_user = cls(
            id=id,
            global_id=global_id,
            username=username,
            first_name=first_name,
            last_name=last_name,
            email=email,
            phone=phone,
            date_created=date_created,
            last_updated=last_updated,
            created_by=created_by,
            updated_by=updated_by,
            active=active,
        )

        get_user_information_response_200_user.additional_properties = d
        return get_user_information_response_200_user

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.asset_attributes import AssetAttributes
from ..models.asset_custom_attributes import AssetCustomAttributes
from ..models.asset_instrumentation_ref import AssetInstrumentationRef
from ..models.asset_status import AssetStatus
from ..models.location import Location
from ..types import UNSET, Unset

T = TypeVar("T", bound="Asset")


@attr.s(auto_attribs=True)
class Asset:
    """
    Attributes:
        id (Union[Unset, str]): Asset ID Example: 272956057382HD3JBSD24.
        active (Union[Unset, bool]):  Example: True.
        name (Union[Unset, str]):  Example: Consumer Asset VINNUMBER12345.
        make (Union[Unset, str]):  Example: Ford.
        model (Union[Unset, str]):  Example: Explorer.
        year (Union[Unset, int]):  Example: 2022.
        vin (Union[Unset, str]):  Example: VINNUMBER12345.
        initial_odometer (Union[Unset, float]):  Example: 9.
        odometer (Union[Unset, float]):  Example: 516.83.
        distance_driven (Union[Unset, float]):  Example: 525.83.
        internal_battery_voltage (Union[Unset, float]):
        attributes (Union[Unset, AssetAttributes]):
        custom_attributes (Union[Unset, AssetCustomAttributes]):
        last_location (Union[Unset, Location]):
        location_last_reported (Union[Unset, str]):  Example: 2022-05-10T03:59:59.999+0000.
        speed (Union[Unset, float]):
        status (Union[Unset, AssetStatus]):  Example: Stopped.
        status_start_date (Union[Unset, str]):  Example: 2022-05-10T03:59:59.999+0000.
        battery_voltage (Union[Unset, float]):  Example: 12.941.
        created_by (Union[Unset, str]):  Example: enterpriseIntegrationServices@spireon.com.
        updated_by (Union[Unset, str]):  Example: username31.
        date_created (Union[Unset, str]):  Example: 2022-05-10T03:59:59.999+0000.
        last_updated (Union[Unset, str]):  Example: 2022-05-10T03:59:59.999+0000.
        instrumentation_ref (Union[Unset, AssetInstrumentationRef]):
    """

    id: Union[Unset, str] = UNSET
    active: Union[Unset, bool] = UNSET
    name: Union[Unset, str] = UNSET
    make: Union[Unset, str] = UNSET
    model: Union[Unset, str] = UNSET
    year: Union[Unset, int] = UNSET
    vin: Union[Unset, str] = UNSET
    initial_odometer: Union[Unset, float] = UNSET
    odometer: Union[Unset, float] = UNSET
    distance_driven: Union[Unset, float] = UNSET
    internal_battery_voltage: Union[Unset, float] = UNSET
    attributes: Union[Unset, AssetAttributes] = UNSET
    custom_attributes: Union[Unset, AssetCustomAttributes] = UNSET
    last_location: Union[Unset, Location] = UNSET
    location_last_reported: Union[Unset, str] = UNSET
    speed: Union[Unset, float] = UNSET
    status: Union[Unset, AssetStatus] = UNSET
    status_start_date: Union[Unset, str] = UNSET
    battery_voltage: Union[Unset, float] = UNSET
    created_by: Union[Unset, str] = UNSET
    updated_by: Union[Unset, str] = UNSET
    date_created: Union[Unset, str] = UNSET
    last_updated: Union[Unset, str] = UNSET
    instrumentation_ref: Union[Unset, AssetInstrumentationRef] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        active = self.active
        name = self.name
        make = self.make
        model = self.model
        year = self.year
        vin = self.vin
        initial_odometer = self.initial_odometer
        odometer = self.odometer
        distance_driven = self.distance_driven
        internal_battery_voltage = self.internal_battery_voltage
        attributes: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.attributes, Unset):
            attributes = self.attributes.to_dict()

        custom_attributes: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.custom_attributes, Unset):
            custom_attributes = self.custom_attributes.to_dict()

        last_location: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.last_location, Unset):
            last_location = self.last_location.to_dict()

        location_last_reported = self.location_last_reported
        speed = self.speed
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        status_start_date = self.status_start_date
        battery_voltage = self.battery_voltage
        created_by = self.created_by
        updated_by = self.updated_by
        date_created = self.date_created
        last_updated = self.last_updated
        instrumentation_ref: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.instrumentation_ref, Unset):
            instrumentation_ref = self.instrumentation_ref.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if active is not UNSET:
            field_dict["active"] = active
        if name is not UNSET:
            field_dict["name"] = name
        if make is not UNSET:
            field_dict["make"] = make
        if model is not UNSET:
            field_dict["model"] = model
        if year is not UNSET:
            field_dict["year"] = year
        if vin is not UNSET:
            field_dict["vin"] = vin
        if initial_odometer is not UNSET:
            field_dict["initialOdometer"] = initial_odometer
        if odometer is not UNSET:
            field_dict["odometer"] = odometer
        if distance_driven is not UNSET:
            field_dict["distanceDriven"] = distance_driven
        if internal_battery_voltage is not UNSET:
            field_dict["internalBatteryVoltage"] = internal_battery_voltage
        if attributes is not UNSET:
            field_dict["attributes"] = attributes
        if custom_attributes is not UNSET:
            field_dict["customAttributes"] = custom_attributes
        if last_location is not UNSET:
            field_dict["lastLocation"] = last_location
        if location_last_reported is not UNSET:
            field_dict["locationLastReported"] = location_last_reported
        if speed is not UNSET:
            field_dict["speed"] = speed
        if status is not UNSET:
            field_dict["status"] = status
        if status_start_date is not UNSET:
            field_dict["statusStartDate"] = status_start_date
        if battery_voltage is not UNSET:
            field_dict["batteryVoltage"] = battery_voltage
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if updated_by is not UNSET:
            field_dict["updatedBy"] = updated_by
        if date_created is not UNSET:
            field_dict["dateCreated"] = date_created
        if last_updated is not UNSET:
            field_dict["lastUpdated"] = last_updated
        if instrumentation_ref is not UNSET:
            field_dict["instrumentationRef"] = instrumentation_ref

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        active = d.pop("active", UNSET)

        name = d.pop("name", UNSET)

        make = d.pop("make", UNSET)

        model = d.pop("model", UNSET)

        year = d.pop("year", UNSET)

        vin = d.pop("vin", UNSET)

        initial_odometer = d.pop("initialOdometer", UNSET)

        odometer = d.pop("odometer", UNSET)

        distance_driven = d.pop("distanceDriven", UNSET)

        internal_battery_voltage = d.pop("internalBatteryVoltage", UNSET)

        _attributes = d.pop("attributes", UNSET)
        attributes: Union[Unset, AssetAttributes]
        if isinstance(_attributes, Unset):
            attributes = UNSET
        else:
            attributes = AssetAttributes.from_dict(_attributes)

        _custom_attributes = d.pop("customAttributes", UNSET)
        custom_attributes: Union[Unset, AssetCustomAttributes]
        if isinstance(_custom_attributes, Unset):
            custom_attributes = UNSET
        else:
            custom_attributes = AssetCustomAttributes.from_dict(_custom_attributes)

        _last_location = d.pop("lastLocation", UNSET)
        last_location: Union[Unset, Location]
        if isinstance(_last_location, Unset):
            last_location = UNSET
        else:
            last_location = Location.from_dict(_last_location)

        location_last_reported = d.pop("locationLastReported", UNSET)

        speed = d.pop("speed", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, AssetStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = AssetStatus(_status)

        status_start_date = d.pop("statusStartDate", UNSET)

        battery_voltage = d.pop("batteryVoltage", UNSET)

        created_by = d.pop("createdBy", UNSET)

        updated_by = d.pop("updatedBy", UNSET)

        date_created = d.pop("dateCreated", UNSET)

        last_updated = d.pop("lastUpdated", UNSET)

        _instrumentation_ref = d.pop("instrumentationRef", UNSET)
        instrumentation_ref: Union[Unset, AssetInstrumentationRef]
        if isinstance(_instrumentation_ref, Unset):
            instrumentation_ref = UNSET
        else:
            instrumentation_ref = AssetInstrumentationRef.from_dict(_instrumentation_ref)

        asset = cls(
            id=id,
            active=active,
            name=name,
            make=make,
            model=model,
            year=year,
            vin=vin,
            initial_odometer=initial_odometer,
            odometer=odometer,
            distance_driven=distance_driven,
            internal_battery_voltage=internal_battery_voltage,
            attributes=attributes,
            custom_attributes=custom_attributes,
            last_location=last_location,
            location_last_reported=location_last_reported,
            speed=speed,
            status=status,
            status_start_date=status_start_date,
            battery_voltage=battery_voltage,
            created_by=created_by,
            updated_by=updated_by,
            date_created=date_created,
            last_updated=last_updated,
            instrumentation_ref=instrumentation_ref,
        )

        asset.additional_properties = d
        return asset

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import ssl
import uuid
from typing import Dict, Union

import attr


@attr.s(auto_attribs=True)
class Client:
    """A class for keeping track of data related to the API"""

    base_url: str = "https://maintenance.spireon.com/v0/rest"
    cookies: Dict[str, str] = attr.ib(factory=dict, kw_only=True)
    headers: Dict[str, str] = attr.ib(
        default={"X-Nspire-Apptoken": "eyJzeXN0ZW1JZCI6MjMsImJyYW5kSWQiOjgxfQ=="},
        kw_only=True,
    )
    timeout: float = attr.ib(5.0, kw_only=True)
    verify_ssl: Union[str, bool, ssl.SSLContext] = attr.ib(True, kw_only=True)

    def get_headers(self) -> Dict[str, str]:
        """Get headers to be used in all endpoints"""
        return {"X-Nspire-Correlationid": str(uuid.uuid4()), **self.headers}

    def with_apptoken(self, apptoken: str) -> "Client":
        """Set X-Nspire-Apptoken header"""
        return self.with_headers({"X-Nspire-Apptoken": apptoken})

    def with_headers(self, headers: Dict[str, str]) -> "Client":
        """Get a new client matching this one with additional headers"""
        return attr.evolve(self, headers={**self.headers, **headers})

    def get_cookies(self) -> Dict[str, str]:
        return {**self.cookies}

    def with_cookies(self, cookies: Dict[str, str]) -> "Client":
        """Get a new client matching this one with additional cookies"""
        return attr.evolve(self, cookies={**self.cookies, **cookies})

    def get_timeout(self) -> float:
        return self.timeout

    def with_timeout(self, timeout: float) -> "Client":
        """Get a new client matching this one with a new timeout (in seconds)"""
        return attr.evolve(self, timeout=timeout)


@attr.s(auto_attribs=True)
class AuthenticatedClient(Client):
    """A Client which has been authenticated for use on secured endpoints"""

    @classmethod
    def from_token(cls, usertoken: str):
        """Create a new client instance from a usertoken"""
        return cls().with_token(usertoken)

    def with_token(self, usertoken: str) -> "Client":
        """Set X-Nspire-Usertoken authentication header"""
        return self.with_headers({"X-Nspire-Usertoken": usertoken})



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.asset_instrumentations_response_200_asset_ref import AssetInstrumentationsResponse200AssetRef
from ..models.asset_instrumentations_response_200_device_ref import AssetInstrumentationsResponse200DeviceRef
from ..types import UNSET, Unset

T = TypeVar("T", bound="AssetInstrumentationsResponse200")


@attr.s(auto_attribs=True)
class AssetInstrumentationsResponse200:
    """
    Attributes:
        id (Union[Unset, str]): Asset ID Example: 272956057382HD3JBSD24.
        asset_id (Union[Unset, str]): Asset ID Example: 272956057382HD3JBSD24.
        device_id (Union[Unset, str]): Device ID Example: 38261738B7DG937BXMQ3.
        device_ref (Union[Unset, AssetInstrumentationsResponse200DeviceRef]):
        asset_ref (Union[Unset, AssetInstrumentationsResponse200AssetRef]):
    """

    id: Union[Unset, str] = UNSET
    asset_id: Union[Unset, str] = UNSET
    device_id: Union[Unset, str] = UNSET
    device_ref: Union[Unset, AssetInstrumentationsResponse200DeviceRef] = UNSET
    asset_ref: Union[Unset, AssetInstrumentationsResponse200AssetRef] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        asset_id = self.asset_id
        device_id = self.device_id
        device_ref: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.device_ref, Unset):
            device_ref = self.device_ref.to_dict()

        asset_ref: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.asset_ref, Unset):
            asset_ref = self.asset_ref.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if asset_id is not UNSET:
            field_dict["assetId"] = asset_id
        if device_id is not UNSET:
            field_dict["deviceId"] = device_id
        if device_ref is not UNSET:
            field_dict["deviceRef"] = device_ref
        if asset_ref is not UNSET:
            field_dict["assetRef"] = asset_ref

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        asset_id = d.pop("assetId", UNSET)

        device_id = d.pop("deviceId", UNSET)

        _device_ref = d.pop("deviceRef", UNSET)
        device_ref: Union[Unset, AssetInstrumentationsResponse200DeviceRef]
        if isinstance(_device_ref, Unset):
            device_ref = UNSET
        else:
            device_ref = AssetInstrumentationsResponse200DeviceRef.from_dict(_device_ref)

        _asset_ref = d.pop("assetRef", UNSET)
        asset_ref: Union[Unset, AssetInstrumentationsResponse200AssetRef]
        if isinstance(_asset_ref, Unset):
            asset_ref = UNSET
        else:
            asset_ref = AssetInstrumentationsResponse200AssetRef.from_dict(_asset_ref)

        asset_instrumentations_response_200 = cls(
            id=id,
            asset_id=asset_id,
            device_id=device_id,
            device_ref=device_ref,
            asset_ref=asset_ref,
        )

        asset_instrumentations_response_200.additional_properties = d
        return asset_instrumentations_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class GetMaintenanceScheduleResponse200ContentItemServiceScheduleItemServiceItemsSeverity(str, Enum):
    NORMAL = "Normal"
    SEVERE = "Severe"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.get_automotive_dealer_profile_response_200_dealership_address import (
    GetAutomotiveDealerProfileResponse200DealershipAddress,
)
from ..models.get_automotive_dealer_profile_response_200_vehicle_valuation import (
    GetAutomotiveDealerProfileResponse200VehicleValuation,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="GetAutomotiveDealerProfileResponse200")


@attr.s(auto_attribs=True)
class GetAutomotiveDealerProfileResponse200:
    """
    Attributes:
        dealership_id (Union[Unset, str]):  Example: 38262739272BDQYAS2GA.
        dealership_name (Union[Unset, str]):  Example: Frank's Toyota Brimington.
        dealership_address (Union[Unset, GetAutomotiveDealerProfileResponse200DealershipAddress]):
        main_phone_number (Union[Unset, str]):  Example: 508193371.
        service_phone_number (Union[Unset, str]):  Example: 508128492.
        primary_franchise (Union[Unset, str]):  Example: Ford.
        website_url (Union[Unset, str]):
        service_url (Union[Unset, str]):
        profile_image_url (Union[Unset, str]):
        vehicle_valuation (Union[Unset, GetAutomotiveDealerProfileResponse200VehicleValuation]):
        repair_order_history_disabled (Union[Unset, bool]):
        map_box_enabled (Union[Unset, bool]):  Example: True.
        allows_iap (Union[Unset, bool]):
        car_pay_enabled (Union[Unset, bool]):  Example: True.
        repair_pal_enabled (Union[Unset, bool]):  Example: True.
    """

    dealership_id: Union[Unset, str] = UNSET
    dealership_name: Union[Unset, str] = UNSET
    dealership_address: Union[Unset, GetAutomotiveDealerProfileResponse200DealershipAddress] = UNSET
    main_phone_number: Union[Unset, str] = UNSET
    service_phone_number: Union[Unset, str] = UNSET
    primary_franchise: Union[Unset, str] = UNSET
    website_url: Union[Unset, str] = UNSET
    service_url: Union[Unset, str] = UNSET
    profile_image_url: Union[Unset, str] = UNSET
    vehicle_valuation: Union[Unset, GetAutomotiveDealerProfileResponse200VehicleValuation] = UNSET
    repair_order_history_disabled: Union[Unset, bool] = UNSET
    map_box_enabled: Union[Unset, bool] = UNSET
    allows_iap: Union[Unset, bool] = UNSET
    car_pay_enabled: Union[Unset, bool] = UNSET
    repair_pal_enabled: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        dealership_id = self.dealership_id
        dealership_name = self.dealership_name
        dealership_address: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.dealership_address, Unset):
            dealership_address = self.dealership_address.to_dict()

        main_phone_number = self.main_phone_number
        service_phone_number = self.service_phone_number
        primary_franchise = self.primary_franchise
        website_url = self.website_url
        service_url = self.service_url
        profile_image_url = self.profile_image_url
        vehicle_valuation: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.vehicle_valuation, Unset):
            vehicle_valuation = self.vehicle_valuation.to_dict()

        repair_order_history_disabled = self.repair_order_history_disabled
        map_box_enabled = self.map_box_enabled
        allows_iap = self.allows_iap
        car_pay_enabled = self.car_pay_enabled
        repair_pal_enabled = self.repair_pal_enabled

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if dealership_id is not UNSET:
            field_dict["dealershipId"] = dealership_id
        if dealership_name is not UNSET:
            field_dict["dealershipName"] = dealership_name
        if dealership_address is not UNSET:
            field_dict["dealershipAddress"] = dealership_address
        if main_phone_number is not UNSET:
            field_dict["mainPhoneNumber"] = main_phone_number
        if service_phone_number is not UNSET:
            field_dict["servicePhoneNumber"] = service_phone_number
        if primary_franchise is not UNSET:
            field_dict["primaryFranchise"] = primary_franchise
        if website_url is not UNSET:
            field_dict["websiteUrl"] = website_url
        if service_url is not UNSET:
            field_dict["serviceUrl"] = service_url
        if profile_image_url is not UNSET:
            field_dict["profileImageUrl"] = profile_image_url
        if vehicle_valuation is not UNSET:
            field_dict["vehicleValuation"] = vehicle_valuation
        if repair_order_history_disabled is not UNSET:
            field_dict["repairOrderHistoryDisabled"] = repair_order_history_disabled
        if map_box_enabled is not UNSET:
            field_dict["mapBoxEnabled"] = map_box_enabled
        if allows_iap is not UNSET:
            field_dict["allowsIAP"] = allows_iap
        if car_pay_enabled is not UNSET:
            field_dict["carPayEnabled"] = car_pay_enabled
        if repair_pal_enabled is not UNSET:
            field_dict["repairPalEnabled"] = repair_pal_enabled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        dealership_id = d.pop("dealershipId", UNSET)

        dealership_name = d.pop("dealershipName", UNSET)

        _dealership_address = d.pop("dealershipAddress", UNSET)
        dealership_address: Union[Unset, GetAutomotiveDealerProfileResponse200DealershipAddress]
        if isinstance(_dealership_address, Unset):
            dealership_address = UNSET
        else:
            dealership_address = GetAutomotiveDealerProfileResponse200DealershipAddress.from_dict(_dealership_address)

        main_phone_number = d.pop("mainPhoneNumber", UNSET)

        service_phone_number = d.pop("servicePhoneNumber", UNSET)

        primary_franchise = d.pop("primaryFranchise", UNSET)

        website_url = d.pop("websiteUrl", UNSET)

        service_url = d.pop("serviceUrl", UNSET)

        profile_image_url = d.pop("profileImageUrl", UNSET)

        _vehicle_valuation = d.pop("vehicleValuation", UNSET)
        vehicle_valuation: Union[Unset, GetAutomotiveDealerProfileResponse200VehicleValuation]
        if isinstance(_vehicle_valuation, Unset):
            vehicle_valuation = UNSET
        else:
            vehicle_valuation = GetAutomotiveDealerProfileResponse200VehicleValuation.from_dict(_vehicle_valuation)

        repair_order_history_disabled = d.pop("repairOrderHistoryDisabled", UNSET)

        map_box_enabled = d.pop("mapBoxEnabled", UNSET)

        allows_iap = d.pop("allowsIAP", UNSET)

        car_pay_enabled = d.pop("carPayEnabled", UNSET)

        repair_pal_enabled = d.pop("repairPalEnabled", UNSET)

        get_automotive_dealer_profile_response_200 = cls(
            dealership_id=dealership_id,
            dealership_name=dealership_name,
            dealership_address=dealership_address,
            main_phone_number=main_phone_number,
            service_phone_number=service_phone_number,
            primary_franchise=primary_franchise,
            website_url=website_url,
            service_url=service_url,
            profile_image_url=profile_image_url,
            vehicle_valuation=vehicle_valuation,
            repair_order_history_disabled=repair_order_history_disabled,
            map_box_enabled=map_box_enabled,
            allows_iap=allows_iap,
            car_pay_enabled=car_pay_enabled,
            repair_pal_enabled=repair_pal_enabled,
        )

        get_automotive_dealer_profile_response_200.additional_properties = d
        return get_automotive_dealer_profile_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.get_maintenance_schedule_response_200_content_item_service_schedule_item_service_items_severity import (
    GetMaintenanceScheduleResponse200ContentItemServiceScheduleItemServiceItemsSeverity,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="GetMaintenanceScheduleResponse200ContentItemServiceScheduleItemServiceItems")


@attr.s(auto_attribs=True)
class GetMaintenanceScheduleResponse200ContentItemServiceScheduleItemServiceItems:
    """
    Attributes:
        severity (Union[Unset, GetMaintenanceScheduleResponse200ContentItemServiceScheduleItemServiceItemsSeverity]):
            Example: Normal.
        action (Union[Unset, str]):  Example: Inspect.
        literal_name (Union[Unset, str]):  Example: Floor Mat Inspect.
    """

    severity: Union[Unset, GetMaintenanceScheduleResponse200ContentItemServiceScheduleItemServiceItemsSeverity] = UNSET
    action: Union[Unset, str] = UNSET
    literal_name: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        severity: Union[Unset, str] = UNSET
        if not isinstance(self.severity, Unset):
            severity = self.severity.value

        action = self.action
        literal_name = self.literal_name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if severity is not UNSET:
            field_dict["severity"] = severity
        if action is not UNSET:
            field_dict["action"] = action
        if literal_name is not UNSET:
            field_dict["literalName"] = literal_name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _severity = d.pop("severity", UNSET)
        severity: Union[Unset, GetMaintenanceScheduleResponse200ContentItemServiceScheduleItemServiceItemsSeverity]
        if isinstance(_severity, Unset):
            severity = UNSET
        else:
            severity = GetMaintenanceScheduleResponse200ContentItemServiceScheduleItemServiceItemsSeverity(_severity)

        action = d.pop("action", UNSET)

        literal_name = d.pop("literalName", UNSET)

        get_maintenance_schedule_response_200_content_item_service_schedule_item_service_items = cls(
            severity=severity,
            action=action,
            literal_name=literal_name,
        )

        get_maintenance_schedule_response_200_content_item_service_schedule_item_service_items.additional_properties = d
        return get_maintenance_schedule_response_200_content_item_service_schedule_item_service_items

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class AssetInstrumentationsResponse200DeviceRefFeaturesItem(str, Enum):
    QUICK_LOCATE = "QUICK_LOCATE"
    NSPIRE_V2 = "NSPIRE_V2"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.address import Address
from ..types import UNSET, Unset

T = TypeVar("T", bound="Location")


@attr.s(auto_attribs=True)
class Location:
    """
    Attributes:
        lat (Union[Unset, float]):  Example: 32.1253.
        lng (Union[Unset, float]):  Example: -72.219482.
        address (Union[Unset, Address]):
    """

    lat: Union[Unset, float] = UNSET
    lng: Union[Unset, float] = UNSET
    address: Union[Unset, Address] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        lat = self.lat
        lng = self.lng
        address: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.address, Unset):
            address = self.address.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if lat is not UNSET:
            field_dict["lat"] = lat
        if lng is not UNSET:
            field_dict["lng"] = lng
        if address is not UNSET:
            field_dict["address"] = address

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        lat = d.pop("lat", UNSET)

        lng = d.pop("lng", UNSET)

        _address = d.pop("address", UNSET)
        address: Union[Unset, Address]
        if isinstance(_address, Unset):
            address = UNSET
        else:
            address = Address.from_dict(_address)

        location = cls(
            lat=lat,
            lng=lng,
            address=address,
        )

        location.additional_properties = d
        return location

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.asset_instrumentations_response_200_device_ref_features_item import (
    AssetInstrumentationsResponse200DeviceRefFeaturesItem,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="AssetInstrumentationsResponse200DeviceRef")


@attr.s(auto_attribs=True)
class AssetInstrumentationsResponse200DeviceRef:
    """
    Attributes:
        id (Union[Unset, str]): Asset ID Example: 272956057382HD3JBSD24.
        serial_number (Union[Unset, str]):  Example: BSHA638QKNB1,.
        features (Union[Unset, List[AssetInstrumentationsResponse200DeviceRefFeaturesItem]]):
        href (Union[Unset, str]):  Example: /devices/38261738B7DG937BXMQ3.
    """

    id: Union[Unset, str] = UNSET
    serial_number: Union[Unset, str] = UNSET
    features: Union[Unset, List[AssetInstrumentationsResponse200DeviceRefFeaturesItem]] = UNSET
    href: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        serial_number = self.serial_number
        features: Union[Unset, List[str]] = UNSET
        if not isinstance(self.features, Unset):
            features = []
            for features_item_data in self.features:
                features_item = features_item_data.value

                features.append(features_item)

        href = self.href

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if serial_number is not UNSET:
            field_dict["serialNumber"] = serial_number
        if features is not UNSET:
            field_dict["features"] = features
        if href is not UNSET:
            field_dict["href"] = href

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        serial_number = d.pop("serialNumber", UNSET)

        features = []
        _features = d.pop("features", UNSET)
        for features_item_data in _features or []:
            features_item = AssetInstrumentationsResponse200DeviceRefFeaturesItem(features_item_data)

            features.append(features_item)

        href = d.pop("href", UNSET)

        asset_instrumentations_response_200_device_ref = cls(
            id=id,
            serial_number=serial_number,
            features=features,
            href=href,
        )

        asset_instrumentations_response_200_device_ref.additional_properties = d
        return asset_instrumentations_response_200_device_ref

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Account")


@attr.s(auto_attribs=True)
class Account:
    """
    Attributes:
        id (Union[Unset, str]): Account ID Example: 1857274837473DSGY71.
        name (Union[Unset, str]):  Example: Consumer Account - e3b77bdd-327d-4b92-a4e6-b43925d1c1a2.
        description (Union[Unset, str]):  Example: Driver account.
        brand_name (Union[Unset, str]):  Example: LoJack.
        brand_code (Union[Unset, str]):  Example: consumerMobile.SkyLink.
        login_enabled (Union[Unset, bool]):  Example: True.
        alternate_ids (Union[Unset, List[str]]):
        date_created (Union[Unset, str]):  Example: 2022-04-22T00:47:02.000+0000.
        last_updated (Union[Unset, str]):  Example: 2022-04-22T00:47:02.000+0000.
        created_by (Union[Unset, str]):  Example: enterpriseIntegrationServices@spireon.com.
        updated_by (Union[Unset, str]):  Example: enterpriseIntegrationServices@spireon.com.
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    brand_name: Union[Unset, str] = UNSET
    brand_code: Union[Unset, str] = UNSET
    login_enabled: Union[Unset, bool] = UNSET
    alternate_ids: Union[Unset, List[str]] = UNSET
    date_created: Union[Unset, str] = UNSET
    last_updated: Union[Unset, str] = UNSET
    created_by: Union[Unset, str] = UNSET
    updated_by: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        description = self.description
        brand_name = self.brand_name
        brand_code = self.brand_code
        login_enabled = self.login_enabled
        alternate_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.alternate_ids, Unset):
            alternate_ids = self.alternate_ids

        date_created = self.date_created
        last_updated = self.last_updated
        created_by = self.created_by
        updated_by = self.updated_by

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if brand_name is not UNSET:
            field_dict["brandName"] = brand_name
        if brand_code is not UNSET:
            field_dict["brandCode"] = brand_code
        if login_enabled is not UNSET:
            field_dict["loginEnabled"] = login_enabled
        if alternate_ids is not UNSET:
            field_dict["alternateIds"] = alternate_ids
        if date_created is not UNSET:
            field_dict["dateCreated"] = date_created
        if last_updated is not UNSET:
            field_dict["lastUpdated"] = last_updated
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if updated_by is not UNSET:
            field_dict["updatedBy"] = updated_by

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        description = d.pop("description", UNSET)

        brand_name = d.pop("brandName", UNSET)

        brand_code = d.pop("brandCode", UNSET)

        login_enabled = d.pop("loginEnabled", UNSET)

        alternate_ids = cast(List[str], d.pop("alternateIds", UNSET))

        date_created = d.pop("dateCreated", UNSET)

        last_updated = d.pop("lastUpdated", UNSET)

        created_by = d.pop("createdBy", UNSET)

        updated_by = d.pop("updatedBy", UNSET)

        account = cls(
            id=id,
            name=name,
            description=description,
            brand_name=brand_name,
            brand_code=brand_code,
            login_enabled=login_enabled,
            alternate_ids=alternate_ids,
            date_created=date_created,
            last_updated=last_updated,
            created_by=created_by,
            updated_by=updated_by,
        )

        account.additional_properties = d
        return account

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['lojack_clients',
 'lojack_clients.automotive',
 'lojack_clients.automotive.api',
 'lojack_clients.automotive.api.default',
 'lojack_clients.automotive.models',
 'lojack_clients.identity',
 'lojack_clients.identity.api',
 'lojack_clients.identity.api.default',
 'lojack_clients.identity.models',
 'lojack_clients.maintenance',
 'lojack_clients.maintenance.api',
 'lojack_clients.maintenance.api.default',
 'lojack_clients.maintenance.models',
 'lojack_clients.services',
 'lojack_clients.services.api',
 'lojack_clients.services.api.default',
 'lojack_clients.services.models']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=21.3.0',
 'httpx>=0.15.4,<0.23.0',
 'python-dateutil>=2.8.0,<3.0.0',
 'uuid']

setup_kwargs = {
    'name': 'lojack-clients',
    'version': '1.0.1',
    'description': "A collection of unofficial client libraries for accessing Spireon's LoJack APIs",
    'long_description': "[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT) [![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\n# lojack-clients\nA collection of client libraries and OpenAPI documentation files for accessing Spireon LoJack's APIs.\n\n## Description\nThis package first and foremost contains unofficial OpenAPI documentation for the APIs that are used for the Spireon LoJack's application. There are four different services that have been identified, each with an API that has its own subdomain. These include `identity`, `automotive`, `maintenance`, and `services`.\n\nThe OpenAPI documentation files can be found under the `openapi` directory. Each service has their own subdirectory which contains only a single `openapi.yaml` file documenting the services' API.\n\nAdditionally, this package contains Python client libraries that have been generated from the OpenAPI documentation using the [openapi-generators/openapi-python-client](https://github.com/openapi-generators/openapi-python-client). Everything inside the `lojack_clients` directory has been generated using the client generator. There are however some minor customizations made to the templates that the generator uses to generate the client classes. These customizations were necessary to make the client work with the APIs' authentication schemes.\n\n### Services\nEach different service has its own purpose with separate endpoints. You will likely primarily be interacting with the `identity` and `services` services. It is likely that there are more services and endpoints that exist, however these ones listed are only the ones that have been discovered so far.\n\n#### identity\nThis service contains a single endpoint which is used for authenticating and generating an access token that can be used with other services.\n\n#### automotive\nThis service contains a single endpoint used to get information about the dealership that your account is associated with.\n\n#### maintenance\nThis service contains two endpoints that relate to maintenance of your car, one for getting repair orders and another for getting a vehicle's suggested maintenance schedule.\n\n#### services\nThis service contains several endpoints which are responsible for the vast majority of the functionality within the LoJack application.\n\n## Usage\nEach service acts as its own library with its own client class.\n\nFirst, you will need to instantiate the identity client which you will use to generate an access token. You should use the `from_login` class factory method to instantiate the identity client.\n\n```python\nfrom lojack_clients.identity import AuthenticatedClient as IdentityClient\n\nidentity_client = IdentityClient.from_login(username, password)\n```\nHere the `AuthenticatedClient` import is aliased as `IdentityClient` to avoid conflicts with `AuthenticatedClient` classes from the other services' client libraries.\n\nAll endpoints require authentication so you will only use the `AuthenticatedClient`. \n\nNext, you will need to call the get_identity_token endpoint to retrieve an access token:\n\n```python\nfrom lojack_clients.identity.models import GetIdentityTokenResponse200\nfrom lojack_clients.identity.api.default import get_identity_token\nfrom lojack_clients.identity.types import Response as IdentityResponse\n\ntoken_response: GetIdentityTokenResponse200 = get_identity_token.sync(client=identity_client)\n# or if you need more info (e.g. status_code)\ntoken_response: Response[GetIdentityTokenResponse200] = get_identity_token.sync_detailed(client=identity_client)\n```\n\nOr do the same thing with an async version:\n\n```python\nfrom lojack_clients.identity.models import GetIdentityTokenResponse200\nfrom lojack_clients.identity.api.default import get_identity_token\nfrom lojack_clients.identity.types import Response as IdentityResponse\n\ntoken_response: GetIdentityTokenResponse200 = await get_identity_token.asyncio(client=identity_client)\n# or if you need more info (e.g. status_code)\ntoken_response: Response[GetIdentityTokenResponse200] = await get_identity_token.asyncio_detailed(client=identity_client)\n```\n\nHere the `Response` import is aliased as `IdentityResponse` to avoid conflicts with `Response` classes from the other services' client libraries.\n\n*Because each of the client libraries are automatically generated, the `Response` classes for each library are all the same and interchangeable.*\n\nYou can then instantiate any of the other three clients using the access token found in the identity response. You should use the `from_token` class factory method to instantiate any of the other three clients.\n\n```python\nfrom lojack_clients.services import AuthenticatedClient as ServicesClient\n\n# if you used the sync or asyncio methods\nservices_client = ServicesClient.from_token(token_response.token)\n\n# or if you used sync_detailed or asyncio_detailed methods\nservices_client = ServicesClient.from_token(token_response.parsed.token)\n```\n\nYou can now use this client to call any of the other endpoints from within the service, similar to how the identity client was used to call the get_identity_token endpoint.\n\n```python\nfrom lojack_clients.services.models import GetAllUserAssetsResponse200\nfrom lojack_clients.services.api.default import get_all_user_assets\nfrom lojack_clients.services.types import Response as ServicesResponse\n\nassets_response: GetAllUserAssetsResponse200 = get_all_user_assets.sync(client=services_client)\n```\n\nJust as the endpoint before, you can use `sync_detailed` if you need more information about the response, and `asyncio` / `asyncio_detailed` to do the same with async calls.\n\nPutting all the steps together, this is what it would look like to authenticate and make a call to retrieve all user assets.\n\n```python\nfrom lojack_clients.identity import AuthenticatedClient as IdentityClient\nfrom lojack_clients.identity.models import GetIdentityTokenResponse200\nfrom lojack_clients.identity.api.default import get_identity_token\nfrom lojack_clients.identity.types import Response as IdentityResponse\nfrom lojack_clients.services import AuthenticatedClient as ServicesClient\nfrom lojack_clients.services.models import GetAllUserAssetsResponse200\nfrom lojack_clients.services.api.default import get_all_user_assets\nfrom lojack_clients.services.types import Response as ServicesResponse\n\nidentity_client = IdentityClient.from_login(username, password)\ntoken_response: GetIdentityTokenResponse200 = get_identity_token.sync(client=identity_client)\n\nservices_client = ServicesClient.from_token(token_response.token)\nassets_response: GetAllUserAssetsResponse200 = get_all_user_assets.sync(client=services_client)\n```\n\nThings to know:\n1. Every path/method combo becomes a Python module with four functions:\n    1. `sync`: Blocking request that returns parsed data (if successful) or `None`\n    1. `sync_detailed`: Blocking request that always returns a `Request`, optionally with `parsed` set if the request was successful.\n    1. `asyncio`: Like `sync` but the async instead of blocking\n    1. `asyncio_detailed`: Like `sync_detailed` by async instead of blocking\n\n1. All path/query params, and bodies become method arguments.\n1. All endpoints can be found under `lojack_clients/{service}/api/default/{endpoint}.py`.\n1. Endpoint file names are generated using the request's `operationId` key in the OpenAPI documentation (converted to snake_case).\n\n## Development\nAll code inside the `lojack_clients` directory is automatically generated. If any changes are made to any of the services' openapi.yaml file, you can update the service's client library by running `./scripts/build_api_clients`. You can pass in specific services that you want to build libraries for, or you can call it without any arguments to build client libraries for all four services. For eg. `./scripts/build_api_clients services identity` would build the libraries for only the `services` and `identity` clients.\n\nCustomized template files for the client libraries exist under `./client-generator/templates/{service}`. If you need to customize some of the generated code you can add or modify Jinja template files there.\n\nOther information about how the code is generated and how it can be configured can be found within the [openapi-generators/openapi-python-client](https://github.com/openapi-generators/openapi-python-client) project on GitHub.\n\n## Contributing\nIf you know of any endpoints or services that aren't disclosed here you're welcome to open up a pull request to add them to the OpenAPI documentation. If there are any other improvements or suggestions, feel free to open a pull request or an issue.\n\n## Disclaimer\nThis project is not affiliated, associated, authorized, endorsed by, or in any way officially connected with Spireon, Inc. The names Spireon and LoJack are trademarks or registered trademarks of Spireon and its subsidiaries. Any related names, marks, emblems and images are registered trademarks of their respective owners. This project has been developed entirely through the use and observation of publicly available information. Use of this project or information within this project comes with no guarantees or warranties. Whenever using or interacting with any services from Spireon, Inc. you must ensure that you are following the Spireon, Inc. terms of service agreement as well as any other agreements that may apply. If you are associated with Spireon, inc and have any questions or concerns, please [submit an issue](https://github.com/scorgn/lojack-clients/issues/new) on the GitHub repository.\n",
    'author': 'Shawn Corrigan',
    'author_email': 'sha.wnja.mesc@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.get_user_information_response_200_account import GetUserInformationResponse200Account
from ..models.get_user_information_response_200_user import GetUserInformationResponse200User
from ..types import UNSET, Unset

T = TypeVar("T", bound="GetUserInformationResponse200")


@attr.s(auto_attribs=True)
class GetUserInformationResponse200:
    """
    Attributes:
        user (Union[Unset, GetUserInformationResponse200User]):
        account (Union[Unset, GetUserInformationResponse200Account]):
    """

    user: Union[Unset, GetUserInformationResponse200User] = UNSET
    account: Union[Unset, GetUserInformationResponse200Account] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        user: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.user, Unset):
            user = self.user.to_dict()

        account: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.account, Unset):
            account = self.account.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if user is not UNSET:
            field_dict["user"] = user
        if account is not UNSET:
            field_dict["account"] = account

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _user = d.pop("user", UNSET)
        user: Union[Unset, GetUserInformationResponse200User]
        if isinstance(_user, Unset):
            user = UNSET
        else:
            user = GetUserInformationResponse200User.from_dict(_user)

        _account = d.pop("account", UNSET)
        account: Union[Unset, GetUserInformationResponse200Account]
        if isinstance(_account, Unset):
            account = UNSET
        else:
            account = GetUserInformationResponse200Account.from_dict(_account)

        get_user_information_response_200 = cls(
            user=user,
            account=account,
        )

        get_user_information_response_200.additional_properties = d
        return get_user_information_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .account import Account
from .address import Address
from .asset import Asset
from .asset_attributes import AssetAttributes
from .asset_custom_attributes import AssetCustomAttributes
from .asset_instrumentation_ref import AssetInstrumentationRef
from .asset_instrumentations_response_200 import AssetInstrumentationsResponse200
from .asset_instrumentations_response_200_asset_ref import AssetInstrumentationsResponse200AssetRef
from .asset_instrumentations_response_200_device_ref import AssetInstrumentationsResponse200DeviceRef
from .asset_instrumentations_response_200_device_ref_features_item import (
    AssetInstrumentationsResponse200DeviceRefFeaturesItem,
)
from .asset_status import AssetStatus
from .command_request_json_body import CommandRequestJsonBody
from .command_request_json_body_command import CommandRequestJsonBodyCommand
from .command_request_json_body_response_strategy import CommandRequestJsonBodyResponseStrategy
from .command_request_response_201 import CommandRequestResponse201
from .command_request_response_201_command import CommandRequestResponse201Command
from .command_request_response_201_response_strategy import CommandRequestResponse201ResponseStrategy
from .coordinates import Coordinates
from .device import Device
from .device_features_item import DeviceFeaturesItem
from .event import Event
from .event_gps_fix_quality import EventGpsFixQuality
from .event_type import EventType
from .geofence import Geofence
from .geofence_event_data import GeofenceEventData
from .geofence_shape import GeofenceShape
from .geofence_shape_type import GeofenceShapeType
from .get_all_user_assets_response_200 import GetAllUserAssetsResponse200
from .get_all_user_devices_response_200 import GetAllUserDevicesResponse200
from .get_specific_user_preferences_namespace import GetSpecificUserPreferencesNamespace
from .get_terms_of_service_response_200 import GetTermsOfServiceResponse200
from .get_user_information_response_200 import GetUserInformationResponse200
from .get_user_information_response_200_account import GetUserInformationResponse200Account
from .get_user_information_response_200_user import GetUserInformationResponse200User
from .get_user_preferences_response_200 import GetUserPreferencesResponse200
from .list_all_events_response_200 import ListAllEventsResponse200
from .list_all_geofences_response_200 import ListAllGeofencesResponse200
from .list_user_preferences_by_namespace_namespace import ListUserPreferencesByNamespaceNamespace
from .list_user_preferences_by_namespace_response_200 import ListUserPreferencesByNamespaceResponse200
from .location import Location
from .null_value import NullValue
from .preference import Preference
from .preference_key import PreferenceKey
from .update_asset_json_body import UpdateAssetJsonBody
from .update_asset_json_body_attributes import UpdateAssetJsonBodyAttributes
from .update_geofence_json_body import UpdateGeofenceJsonBody
from .update_specific_user_preferences_namespace import UpdateSpecificUserPreferencesNamespace
from .user import User
from .user_security_groups_item import UserSecurityGroupsItem



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.coordinates import Coordinates
from ..models.geofence_shape_type import GeofenceShapeType
from ..types import UNSET, Unset

T = TypeVar("T", bound="GeofenceShape")


@attr.s(auto_attribs=True)
class GeofenceShape:
    """
    Attributes:
        type (Union[Unset, GeofenceShapeType]):
        point1 (Union[Unset, Coordinates]):
        point2 (Union[Unset, Coordinates]):
    """

    type: Union[Unset, GeofenceShapeType] = UNSET
    point1: Union[Unset, Coordinates] = UNSET
    point2: Union[Unset, Coordinates] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        point1: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.point1, Unset):
            point1 = self.point1.to_dict()

        point2: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.point2, Unset):
            point2 = self.point2.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if point1 is not UNSET:
            field_dict["point1"] = point1
        if point2 is not UNSET:
            field_dict["point2"] = point2

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _type = d.pop("type", UNSET)
        type: Union[Unset, GeofenceShapeType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = GeofenceShapeType(_type)

        _point1 = d.pop("point1", UNSET)
        point1: Union[Unset, Coordinates]
        if isinstance(_point1, Unset):
            point1 = UNSET
        else:
            point1 = Coordinates.from_dict(_point1)

        _point2 = d.pop("point2", UNSET)
        point2: Union[Unset, Coordinates]
        if isinstance(_point2, Unset):
            point2 = UNSET
        else:
            point2 = Coordinates.from_dict(_point2)

        geofence_shape = cls(
            type=type,
            point1=point1,
            point2=point2,
        )

        geofence_shape.additional_properties = d
        return geofence_shape

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
